<?php

namespace Drupal\mcapi_credcom\EventSubscriber;

use Drupal\Core\Utility\Error;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use CreditCommons\Exceptions\CCFailure;

/**
 * throws 500 errors to credcom paths.
 */
class ExceptionSubscriber extends \Drupal\Core\EventSubscriber\HttpExceptionSubscriberBase {

  /**
   * Handles exceptions for this subscriber.
   *
   * @param \Symfony\Component\HttpKernel\Event\ExceptionEvent $event
   *   The event to process.
   */
  public function onException(ExceptionEvent $event) {
    $exception = $event->getThrowable();
    if (str_contains($exception->getMessage(), '/credcom/')){
      $status_code = $exception->getStatusCode();
      // @message, %type, %function, %file, %line, 'backtrace', @backtrace_string, exception
      $error = Error::decodeException($exception);
      $backtrace_exception = $exception;
      while ($backtrace_exception->getPrevious()) {
        $backtrace_exception = $backtrace_exception->getPrevious();
      }
      $backtrace = $backtrace_exception->getTrace();
      \Drupal::logger('mcapi_credcom')->error('@message', ['@message' => $error['@message']]);
      $content =  new CCFailure($error['@message']);
      $response = new JsonResponse(['errors' => [$content]], 500, ['Content-Type' => "application/json"]);
      $event->setResponse($response);
    }
    else {
      parent::onException($event);
    }
  }

  protected function getHandledFormats() {
    return ['html', 'json'];
  }


  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array {
    // Run as the final (very late) KernelEvents::EXCEPTION subscriber.
    $events[KernelEvents::EXCEPTION][] = ['onException', 257];
    return $events;
  }
}
