<?php

namespace Drupal\mcapi_credcom\EventSubscriber;

use Drupal\mcapi\Event\TransactionSaveEvent;
use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Example subscriber showing the two McapiEvents.
 */
class TransactionSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritDoc}
   */
  public static function getSubscribedEvents() : array{
    return [
      McapiEvents::ASSEMBLE => ['requireSignatures', -10],
      //McapiEvents::ACTION => ['onTransactionAction']
    ];
  }

  /**
   * Notify the trunkward node of a change in state.
   *
   * @param TransactionSaveEvent $event
   *   $event->getArguments() yields form_values, old_state and operation name,
   *    or action. $events->getSubject() gives the transaction.
   * @param string $eventName
   *   The machine name of the event.
   * @param ContainerAwareEventDispatcher $container
   *   The container.
   */
  public function onTransactionAction(TransactionSaveEvent $event, $eventName, ContainerAwareEventDispatcher $container) {
    $transaction = $event->getSubject();
    [$form_values, $old_state, $op_name] = $event->getArguments();
    switch ($op_name) {
      case 'save':
        $dest_cc_state = $transaction->credCom->workflow->startState();
      default:

    }
    try {
      $transaction->set('state', $dest_state);
    }
    catch (Throwable $e) {
      \Drupal::messenger()->addError(t(
        'There was a problem saving the transaction to the network: @message',
        ['@message' => $e->getMessage()]
      ));
      \Drupal::logger('mcapi_credcom')->notice('@notice', ['@notice' => $e->getMessage()]);
    }
    $event->setMessage("TransactionSubscriber responded to $eventName");
  }

  /**
   * Signatures module doesn't handle remote transactions.
   */
  public function requireSignatures(TransactionAssembleEvent $event) {
    $transaction = $event->getSubject();
    if ($transaction->credCom and !$transaction->isChild) {
      $wf = $transaction->credCom->workflow;
      if($wf->startState() == 'P') {
        $wallet_id = \Drupal::config('mcapi_credcom.settings')->get('wallet_id');
        if ($wf->isBill()) {
          $sig_wallet_items = $transaction->payer;
        }
        elseif ($wf->isCredit()) {
          $sig_wallet_items = $transaction->payee;
        }
        else {
          throw new \Exception('mcapi_credcom does not support thirdparty signed transactions');
        }
        if ($sig_wallet_items->target_id == $wallet_id) {
          $transaction->signatures[0] = 0;
        }
        else {
          $transaction->signatures[$sig_wallet_items->entity->getOwnerId()] = 0;
        }
        $transaction->state->setValue('pending');
      }
    }
  }

}
