<?php

namespace Drupal\mcapi_credcom\Form;

use Drupal\mcapi_credcom\TrunkwardNodeRequester;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use GuzzleHttp\Client;

/**
 * @deprecated
 */
class SettingsForm extends ConfigFormBase {

  private $entityTypeManager;
  private $traded;
  private $allWorkflows = [];
  private \Drupal\Core\Config\Config $config;

  /**
   * {@inheritDoc}
   */
  public function getFormID() {
    return 'mcapi_credcom_settings_form';
  }

  /**
   * Constructor
   *
   * @param $config_factory
   * @param EntityTypeManager $entity_type_manager
   * @param TrunkwardNodeRequester $trunkward_node
   */
  public function __construct($config_factory, EntityTypeManager $entity_type_manager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entity_type_manager;
    $this->config = $config_factory->get('mcapi_credcom.settings');
    $this->traded = (bool)$entity_type_manager->getStorage('mcapi_transaction')
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', 'remote')
      ->execute();
  }

  static public function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['trunkward_node_url'] = [
      '#title' => t('Trunkward node URL'),
      '#description' => t('Must be https to protect the login credentials.'),
      '#type' => 'textfield',
      '#default_value' => $this->config->get('trunkward_node_url') ?? '',
      '#placeholder' => 'https://',
      '#pattern' => 'https?:\/\/[a-z/.\-0-9]+', // https enforced except during development
      '#required' => TRUE,
      '#disabled' => (bool)$this->config->get('trunkward_node_url'),
      '#weight' => 2
    ];

    $path = $this->config->get('trunkward_node_path');
    if (!$path and $url = $this->config->get('trunkward_node_url')) {
      $path = $this->checkNode($url, $form_state);
    }

    $form['trunkward_node_path'] = [
      '#title' => $this->t('Absolute path of trunkward node'),
      '#description' => $this->t('populated automatically when the trunkward account is created.'),
      '#type' => 'textfield',
      '#default_value' => $path,
      '#weight' => 3,
      '#disabled' => TRUE
    ];
    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    $form_state->setValue(
      'trunkward_node_path',
      $this->checkNode($form_state->getValue('trunkward_node_url'), $form_state)
    );
  }

  public function checkNode($url, FormStateInterface $form_state) : string {
    $client = new Client(['base_uri' => $url]);
    try {
      $result = $client->get('/about')->getBody();
    }
    catch(\Throwable $e) {
      $form_state->setErrorByName('trunkward_node_path', $e->getMessage());
      return '';
    }
    $payload = json_decode(strval($result));
    return implode('/', $payload->data->absolute_path);
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $trunkward_node_name = $form_state->getValue('trunkward_node_path');
    $pos = strrpos($trunkward_node_name,  '/');
    if(is_int($pos)) {
      $trunkward_node_name = substr($trunkward_node_name, $pos +1);
    }
    $this->configFactory->getEditable('mcapi_credcom.settings')
      ->set('trunkward_node_path', $form_state->getValue('trunkward_node_path'))
      ->set('trunkward_node_url', $form_state->getValue('trunkward_node_url'))
      ->set('trunkward_node_name', $trunkward_node_name)
      ->save();
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames() {
    return ['mcapi_credcom.settings'];
  }

  /**
   * Make a transaction form for each selected remote workflow
   */
  function rebuildFirstpartyForms(array $active_workflow_names) {
    $form_display_ids = $this->entityTypeManager->getStorage('entity_form_display')
      ->getQuery()
      ->condition('id', 'mcapi_transaction.mcapi_transaction.remote_', 'STARTS_WITH')
      ->execute();
    // delete unused form displays.
    foreach ($form_display_ids as $form_display_id) {
      $wf_name = substr($form_display_id, strlen('mcapi_transaction.mcapi_transaction.remote_'));
      if (empty($active_workflow_names[$wf_name])) {
        if ($form_display = EntityFormDisplay::load($form_display_id)) {
          $this->messenger()->addStatus($this->t('Deleted FormDisplay @name', ['@name' => $form_display->id()]));
          $form_display->delete();
        }
      }
      unset($active_workflow_names[$wf_name]);
    }
    // Create new form displays
    foreach ($active_workflow_names as $wf_name) {
      if (EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.remote_'.$wf_name)) {
        return;
      }
      // Make a new EntityFormDisplay.
      // The EntityFormMode already exists, now make the EntityFormDisplay.
      $wf = $this->allWorkflows[$wf_name];
      $id = 'remote_'.$wf->id;
      $form_display_values = EntityFormDisplay::load('mcapi_transaction.mcapi_transaction.default')->toArray();
      $form_display_values['mode'] = $id;
      $form_display_values['id'] = 'mcapi_transaction.mcapi_transaction.'.$id;
      unset($form_display_values['uuid']);
      // Create a new entity with the original values.
      $form_display = EntityFormDisplay::create($form_display_values);
      $form_display->setComponent(in_array('payer', $wf->creation->by) ? 'payee' : 'payer',
        [
          'region' => 'content',
          'type' => 'credcom_wallet',
          'settings' => ['size' => 30, 'placeholder' => 'path/to/account'],
          'weight' => 3
        ]
      )->setComponent('worth',
        [
          'region' => 'content',
          'type' => 'worth',
          'settings' => ['exclude' => FALSE, 'currencies' => [$this->config->get('currency_id')]],
          'weight' => 3
        ]
      )->save();
      $url = Url::fromRoute('entity.entity_form_display.mcapi_transaction.form_mode', ['form_mode_name' => $id]);
      $this->messenger()->addStatus($this->t(
        'Now go and <a href=":url">edit</a> the new remote "@label" form.',
        [':url' => $url->toString(), '@label' => $wf->id]
      ));
    }

  }


}
