<?php

namespace Drupal\mcapi_credcom;

use Drupal\mcapi_credcom\Functions;
use Drupal\Core\Access\AccessResult;
use Symfony\Component\HttpFoundation\Request;
use CreditCommons\Exceptions\HashMismatchFailure;

/**
 * Access check for requests from trunkwards nodes.
 */
class HashChecker implements \Drupal\Core\Routing\Access\AccessInterface{


  /**
   * This is actually setting up drupal to respond like a credit commons node.
   */
  function __construct(\Symfony\Component\HttpFoundation\RequestStack $request_stack) {
    // These remain in place for the whole request, which is always an API request from the trunkward node
    set_exception_handler(['\Drupal\mcapi_credcom\Functions', 'CredcomExceptionHandler']);
    set_error_handler(['\Drupal\mcapi_credcom\Functions', 'CredcomErrorHandler']);
  }

  /**
   * Checks the hash passed in the headers against the hash in the database.
   *
   * @param RequestStack $request
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   */
  public function access(Request $request) {
    global $error_context;
    $currency = $request->attributes->get('mcapi_currency');
    $curr_settings = $currency->getThirdPartySettings('mcapi_credcom');
    Functions::createErrorContext($currency);
    $local_hash = Functions::getlastHash($curr_settings['node_name']) or $local_hash = $curr_settings['trunkward_node']['name'];
    $remote_hash = $request->headers->get('last-hash');
    if ($local_hash == $remote_hash) {
      return AccessResult::allowed();
    }
    //return AccessResult::allowed();
    return AccessResult::forbidden('HashMismatchFailure'); // temp
  }

}
