<?php

namespace Drupal\mcapi_credcom\Plugin\Action;

use Drupal\mcapi_credcom\Functions;
use Drupal\mcapi\Plugin\TransactionActionBase;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Move a remote transaction to the Completed state
 *
 * @Action(
 *   id = "mcapi_transaction.remote_finalise_action",
 *   label = @Translation("Finalise a remote transaction"),
 *   type = "mcapi_transaction"
 * )
 */
class Finalise extends TransactionActionBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $elements = parent::buildConfigurationForm($form, $form_state);
    unset($elements['states'], $elements['access']);
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function access($transaction, AccountInterface $account = NULL, $return_as_object = FALSE) {
    $access = FALSE;

    if ($transaction->state->target_id == 'pending' && Functions::isRemote($transaction)) {
      $uid = $account->id();
      if ($account->hasPermission('manage mcapi') or (isset($transaction->signatures[$uid]) and $transaction->signatures[$uid] == 0)) {
        $access = TRUE;
      }
    }
    return $return_as_object ? AccessResult::allowedIf($access) : $access;
  }

  /**
   * {@inheritdoc}
   */
  public function execute($object = NULL) {
    try {
      // Ensure all signatures are signed.
      mcapi_signature_service($object)->signOff();
    }
    catch (Throwable $e) {
      \Drupal::messenger()->addError(t(
        'There was a problem saving the transaction to the network: @message',
        ['@message' => $e->getMessage()]
      ));
      \Drupal::logger('mcapi_credcom')->notice('@notice', ['@notice' => $e->getMessage()]);
    }

    parent::execute($object);

  }

}
