<?php

namespace Drupal\mcapi_credcom\Plugin\Derivative;

use Drupal\Core\Config\Entity\Query\Query;
use Drupal\Core\Entity\Entity\EntityFormMode;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Component\Plugin\Derivative\DeriverBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides local actions for transaction forms.
 */
class TransactionFormTasks extends DeriverBase implements ContainerDeriverInterface {

  private $formModes;

  /**
   * {@inheritDoc}
   */
  public function __construct(Query $form_display_query) {
    $form_ids = $form_display_query
      ->condition('targetEntityType', 'mcapi_transaction')
      ->condition('id', 'mcapi_transaction.default', '<>')
      ->accessCheck(FALSE)
      ->execute();
    $this->formModes = array_filter(
      EntityFormMode::loadMultiple($form_ids),
      function ($form_mode) {return $form_mode->getThirdPartySetting('mcapi_forms', 'transaction_type') == 'remote';}
    );
    // Put the forms in alphabetical order of the menu title
    usort(
      $this->formModes,
      function ($a, $b){return $a->getThirdpartySetting('mcapi_forms', 'menu_title') <=> $b->getThirdpartySetting('mcapi_forms', 'menu_title');}
    );
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('entity_type.manager')->getStorage('entity_form_mode')->getQuery()
    );
  }

  /**
   * {@inheritDoc}
   * Make a default tab for the main views
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $tabs = [];
    $w=0;
    foreach ($this->formModes as $form_mode) {
      $settings = $form_mode->getThirdpartySettings('mcapi_forms');
      $tab_title = $settings['menu_title'];
      $form_id = $form_mode->id();
      $mode_name = substr($form_id, strpos($form_id, '.')+1);
      $this->derivatives[$form_id] = [
        'title' => $tab_title,
        'description' => '',
        'route_name' => 'mcapi.1stparty.' . $mode_name,
        'parent_id' => 'mcapi_credcom.remote',
        'weight' => $w++,
      ] + $base_plugin_definition;
    }
    return $this->derivatives;
  }

}
