<?php

namespace Drupal\mcapi_mass;

use Drupal\mcapi\Event\McapiEvents;
use Drupal\mcapi\Event\TransactionAssembleEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Example
 */
class TransactionSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() : array {
    return [
      McapiEvents::ASSEMBLE => ['onmakeChildren']
    ];
  }

  /**
   * Charge a fee in the first listed currency.
   *
   * @param TransactionAssembleEvent $event
   *   The event.
   *
   * @note The only performance trial I've done is 511 children with 128MB RAM on a fairly complex site.
   */
  public function onMakeChildren(TransactionAssembleEvent $event) {
    $parent_transaction = $event->getSubject();
    if ($parent_transaction->type->target_id == 'mass') {
      foreach ($parent_transaction->many as $wallet_id) {
        $values = $parent_transaction->toArray();
        unset($values['uuid']);
        $values[$parent_transaction->manyField][0]['target_id'] = $wallet_id;
        if ($values['payer'][0]['target_id'] == $values['payee'][0]['target_id']) {
          continue;
        }
        $event->addChildTransaction($values);
      }
      unset($parent_transaction->many);
    }
  }

}
