<?php

namespace Drupal\mcapi\Entity\Storage;

use Drupal\mcapi\Entity\Wallet;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Storage controller for wallet entity.
 */
class WalletStorage extends SqlContentEntityStorage implements WalletStorageInterface {

  /**
   * {@inheritdoc}
   */
  public function getQueryServiceName() {
    return 'mcapi_wallet.query.sql';
  }

  /**
   * {@inheritdoc}
   */
  public static function walletsOf(ContentEntityInterface $entity, $load = FALSE) : array {
    // Access check is that the owner is active not blocked.
    // Also that it runs hook_query_mcapi_wallet_access_alter
    $wids = \Drupal::entityQuery('mcapi_wallet')->accessCheck(FALSE)
      ->condition('owner', $entity->id())
      ->addMetaData('mcapi_wallet_selection_handler', 1)
      ->execute();
    return $load ?
      Wallet::loadMultiple($wids) :
      $wids;
  }

  /**
   * Return the first wallet held by a given entity.
   *
   * @param ContentEntityBase $entity
   *   Defaults to the current user.
   *
   * @return int
   *   0 if no wallets are held
   */
  static function firstWalletOfEntity(AccountInterface $entity) : int {
    if ($wids = self::walletsOf($entity)) {
      return reset($wids);
    }
    return 0;
  }

  static function hasWallet(AccountInterface $entity) : bool {
    return (bool)static::walletsOf($entity, FALSE);
  }

}

