<?php

namespace Drupal\mcapi\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\user\UserInterface;

/**
 * Provides an interface defining an exchange entity.
 */
interface WalletInterface extends ContentEntityInterface {

  /**
   * Get the trading stats for all currencies allowed or used in the wallet.
   *
   * @param bool $include_empty
   *   Return empty stats for unused currencies.
   *
   * @return array
   *   Arrays of statistics, keyed by currency id.
   *
   * @see \Drupal\mcapi\Entity\Storage\TransactionStorageInterface::summaryData()
   */
  public function getAllStats($include_empty = FALSE) : array;

  /**
   * Get the standard trading stats.
   *
   * @param string $curr_id
   *   the entity key for the mcapi_currency configEntity.
   *
   * @return array
   *   Array of statistics, keyed by stat name.
   *
   * @see $this->getStat()
   */
  public function getStats(string $curr_id) : \stdClass;

  /**
   * Get the balance for a particular currency
   *
   * @param string $curr_id
   *
   * @return string
   *   The formatted balance
   */
  public function balance($curr_id, $display = CurrencyInterface::DISPLAY_NORMAL, $linked = TRUE) : string;

  /**
   * Get the balance(s) of the current wallet, in worth format.
   *
   * @param string $stat
   *   Which stat we want to receive.
   *
   * @return array
   *   Worth values in no particular order, each with curr_id and (raw) value.
   */
  public function getStatAll($stat = 'balance') : array;

  /**
   * Check whether a wallet has ever been used. i.e. whether the journal
   * references it.
   *
   * @return bool
   *   TRUE if the wallet has never been used.
   */
  public function isEmpty() : bool;

}
