<?php

namespace Drupal\mcapi;

use Drupal\Component\Utility\Html;
use Drupal\mcapi\Entity\Currency;

/**
 * Utility class for community accounting.
 */
class Mcapi {

  /**
   * Get a list of entity labels keyed by entity ID suitable for a select box.
   * @param string $entity_type_id
   * @return string[]
   *
   * @note this is only used for mcapi_state, mcapi_type and mcapi_currency ATM
   */
  static function entityOptions(string $entity_type_id) : array {
    $entities = \Drupal::entityTypeManager()->getStorage($entity_type_id)->loadMultiple();
    $contextRepo = \Drupal::service('entity.repository');
    foreach ($entities as $entity) {
      $label = $contextRepo->getTranslationFromContext($entity)->label();
      $list[$entity->id()] = Html::escape($label);
    }
    return $list;
  }

  /**
   * Get a list of all the tokens on the transaction entity.
   *
   * @note This really needs replacing with a core function.
   */
  public static function tokenHelp() : string {
    foreach (array_keys(\Drupal::Token()->getInfo()['tokens']['xaction']) as $token) {
      $tokens[] = "[xaction:$token]";
    }
    return implode(', ', $tokens);
  }

  /**
   *
   * @param Currency $currency
   * @param array|null $conditions
   * @return type
   */
  public static function tradingStats(Currency $currency, ?array $conditions = []) {
    $balances_asc = $trades = $volumes = $incomes = $spending = [];
    $rows = \Drupal::entityTypeManager()
      ->getStorage('mcapi_transaction')
      ->ledgerStateByWallet($currency->id(), $conditions);
    foreach ($rows as $row) {
      $balances_asc[$row->wid] = (int)$row->balance;
      $trades[$row->wid] = $row->trades;
      $volumes[$row->wid] = (int)$row->income + (int)$row->expenditure;
      $incomes[$row->wid] = (int)$row->income;
      $spending[$row->wid] = (int)$row->expenditure;
    }
    return [$balances_asc, $trades, $volumes, $incomes, $spending];
  }

}
